@@ -20,54 +20,43 @@ from . import OUTPUT_DIR_STR, NAME_STR, FILE_STR, JSON_STR, RATE_STR, MESSAGES_S
 
 # ------------------------------------------------------------------------------
 
-_record_thread = None
-
-# ------------------------------------------------------------------------------
-
 def config(**kwargs):
 
     serial_port = kwargs.get(SERIAL_PORT_STR, None)
-    conn_config = receiver.detect_connection(serial_port=serial_port)
+    baudrate = kwargs.get(BAUD_RATE_STR, None)
+    config_file = kwargs.get(FILE_STR, None)
+    json_file = kwargs.get(JSON_STR, None)
 
-    serial_port = conn_config[SERIAL_PORT_STR]
-    baudrate = conn_config[BAUD_RATE_STR]
+    ublox_receiver = receiver.Receiver(serial_port=serial_port, baudrate=baudrate)
 
-    logger.info(f'Attempting to configure device in serial port [ {serial_port},{baudrate} ]')
+    if config_file:
+        logger.info(f'Configuring device from file [ {config_file} ]')
+        ublox_receiver.submit_configuration_from_ucenter_file(config_file)
 
-    with serial.Serial(serial_port, baudrate) as stream:
-        
-        config_file = kwargs.get(FILE_STR, None)
-        json_file = kwargs.get(JSON_STR, None)
-        if config_file:
-            logger.info(f'Configuring device from file [ {config_file} ]')
-            with open(config_file, 'r') as fh:
-                ublox_config.set_from_ucenter_file(stream, fh)
-
-        elif json_file:
-            logger.info(f'Configuring device from JSON file [ {json_file} ]')
-            with open(json_file, 'r') as fh:
-                ublox_config.set_from_json_file(stream, fh) 
-
-        else:
-            config_dict = {}
-
-            if RATE_STR in kwargs:
-                config_dict[constants.RATE_STR] = {
-                    constants.MEASUREMENTS_STR : kwargs[RATE_STR],
-                    constants.SOLUTION_STR : kwargs[RATE_STR] * 5
-                }
-
-            logger.info(f'Configuring device from command line options: {config_dict}')
-            ublox_config.set_from_dict(stream, config_dict)
+    elif json_file:
+        logger.info(f'Configuring device from JSON file [ {json_file} ]')
+        with open(json_file, 'r') as fh:
+            ublox_receiver.submit_configuration(json.loads(fh.read()))
 
+    else:
+        config_dict = {}
 
+        if RATE_STR in kwargs:
+            config_dict[constants.RATE_STR] = {
+                constants.MEASUREMENTS_STR : kwargs[RATE_STR],
+                constants.SOLUTION_STR : kwargs[RATE_STR] * 5
+            }
 
+        logger.info(f'Configuring device from command line options: {config_dict}')
+        ublox_receiver.submit_configuration(config_dict)
+ 
 # ------------------------------------------------------------------------------
 
 def detect(serial_port=None, baudrate=None, messages=False):
 
-    config = receiver.detect_config(serial_port=serial_port, baudrate=baudrate, messages=messages)
+    ublox_receiver = receiver.Receiver(serial_port=serial_port, baudrate=baudrate, detect_messages=messages)
 
+    config = ublox_receiver.get_configuration()
     if not config:
         return False
 
@@ -82,13 +71,15 @@ def detect(serial_port=None, baudrate=None, messages=False):
 def reset(**kwargs):
 
     serial_port = kwargs.get(SERIAL_PORT_STR, None)
-    if not serial_port:
-        logger.critical('Need to specify the port to be reset')
-        return False
-    else:
-        logger.info(f'Resetting device in [ {serial_port} ]')
-        receiver.reset(serial_port)
-        return True
+    baudrate = kwargs.get(BAUD_RATE_STR, None)
+    ublox_receiver = receiver.Receiver(serial_port=serial_port, baudrate=baudrate)
+    ublox_receiver.reset()
+
+    return True
+
+# ------------------------------------------------------------------------------
+
+_receiver = None
 
 # ------------------------------------------------------------------------------
 
@@ -106,10 +97,10 @@ def record(**kwargs):
         'receiver_name': kwargs.get(NAME_STR, 'UBLX')
     }
 
-    global _record_thread
-    _record_thread = receiver.Recorder(**recorder_config)
+    global _receiver
+    _receiver = receiver.Receiver(**recorder_config)
 
-    _record_thread.start()
+    _receiver.start()
 
 
 # ------------------------------------------------------------------------------
@@ -117,8 +108,8 @@ def record(**kwargs):
 def interruption_handler(sig, frame):
     logger.info('You pressed Ctrl+C, gracefully closing files and serial streams')
 
-    global _record_thread
-    _record_thread.stop()
+    global _receiver
+    _receiver.stop()
 
     sys.exit(0)
 
