@@ -51,7 +51,8 @@ class Receiver(threading.Thread):
         self.current_epoch_suffix = None
         self.fout = None
         self.tempdir = None
-        self.do_recording = False
+
+        self.do_recording = threading.Event()
 
     # ---
 
@@ -136,9 +137,7 @@ class Receiver(threading.Thread):
 
     def run(self):
 
-        logger.debug('Starting recording thread')
-
-        self.do_recording = True
+        self.do_recording.set()
 
         logger.info("Writing ubx data from _serial_stream [ {} ] for receiver [ {} ] to files "
                     "in folder [ {} ], with [ {} ] periodicity".format(
@@ -148,61 +147,66 @@ class Receiver(threading.Thread):
         
         logger.debug('Partial files will be written in this temporary folder [ {} ]'.format(self.tempdir))
 
-        self.serial_stream_for_recording = serial.Serial(self.serial_port, self.baudrate)
-        if self.serial_stream_for_recording:
+        while True:
 
-            incoming_epoch_suffix = None
-            num_packets = 0
-            for packet in core.ublox_packets(self.serial_stream_for_recording, core.TIMEOUT_DEF):
+            self.do_recording.wait()
 
-                try:
-                    packet_type, parsed_packet = core.parse(packet)
-                except Exception:
-                    continue
-                
-                if packet_type == core.PacketType.RXM_RAWX:
-                    tow = parsed_packet.rcvTow
-                    week = parsed_packet.week
-                    epoch = weektow_to_datetime(tow, week)
-                    num_packets += 1
-                    incoming_epoch_suffix = self.slice.build_rinex3_epoch(epoch)
-                    if num_packets % 1000 == 0:
-                        logger.debug("U-blox RXM_RAWX packet found! Epoch suffix {} , num_packets = {}".format(incoming_epoch_suffix, num_packets))
+            with serial.Serial(self.serial_port, self.baudrate) as serial_stream:
 
-                if not incoming_epoch_suffix:
-                    continue
+                incoming_epoch_suffix = None
+                num_packets = 0
+                for packet in core.ublox_packets(serial_stream, core.TIMEOUT_DEF):
 
-                if not self.current_epoch_suffix or self.current_epoch_suffix != incoming_epoch_suffix:
+                    try:
+                        packet_type, parsed_packet = core.parse(packet)
+                    except Exception:
+                        continue
+                    
+                    if packet_type == core.PacketType.RXM_RAWX:
+                        tow = parsed_packet.rcvTow
+                        week = parsed_packet.week
+                        epoch = weektow_to_datetime(tow, week)
+                        num_packets += 1
+                        incoming_epoch_suffix = self.slice.build_rinex3_epoch(epoch)
+                        if num_packets % 1000 == 0:
+                            logger.debug("U-blox RXM_RAWX packet found! Epoch suffix {} , num_packets = {}".format(incoming_epoch_suffix, num_packets))
 
-                    self.__close_and_save_partial_file__()
+                    if not incoming_epoch_suffix:
+                        continue
 
-                    self.current_epoch_suffix = incoming_epoch_suffix
+                    if not self.current_epoch_suffix or self.current_epoch_suffix != incoming_epoch_suffix:
 
-                    filename = os.path.join(self.tempdir,'{}_{}.ubx'.format(self.receiver_name, self.current_epoch_suffix))
-                    self.fout = open(filename, 'wb')
-                    logger.info("Created new data file [ {} ]".format(os.path.basename(filename)))
+                        self.__close_and_save_partial_file__()
 
-                if not self.do_recording:
-                    break
+                        self.current_epoch_suffix = incoming_epoch_suffix
 
-                self.fout.write(core.PREAMBLE)
-                self.fout.write(packet)
+                        filename = os.path.join(self.tempdir,'{}_{}.ubx'.format(self.receiver_name, self.current_epoch_suffix))
+                        self.fout = open(filename, 'wb')
+                        logger.info("Created new data file [ {} ]".format(os.path.basename(filename)))
+
+                    if not self.do_recording.is_set():
+                        break
+
+                    self.fout.write(core.PREAMBLE)
+                    self.fout.write(packet)
 
     # ---
 
     def stop(self):
 
-        if self.serial_stream_for_recording:
-            self.serial_stream_for_recording.close()
-            serial_port = self.connection_config.get(SERIAL_PORT_STR, None)
-            logger.info('Closed serial stream from [ {} ]'.format(serial_port))
+        if self.is_recording_ongoing():
+            self.__close_and_save_partial_file__()
 
-        self.__close_and_save_partial_file__()
+            if self.tempdir:
+                shutil.rmtree(self.tempdir)
+                
+        self.do_recording.clear()
 
-        if self.tempdir:
-            shutil.rmtree(self.tempdir)
+    def resume(self):
+        self.do_recording.set()
 
-        self.do_recording = False
+    def is_recording_ongoing(self):
+        return self.do_recording.is_set()
 
     # ---
 
